"use client";

import DataTable from "@/components/tables/DataTable";
import { columnsDataBlocks } from "../variables/columnsBlock";
import { TableData } from "@/variables/columnsData";
import { Button, Flex, FormControl, FormLabel, Icon, Modal, ModalBody, ModalCloseButton, ModalContent, ModalFooter, ModalHeader, ModalOverlay, Radio, RadioGroup, Stack, Text, useColorModeValue, useDisclosure, useToast } from "@chakra-ui/react";
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";
import FormComponent from "@/components/form/Form";
import { InputWrapper, RadioGroupWrapper } from "@/components/fields/Fields";
import { useEffect, useMemo, useRef, useState } from "react";
import prepareValidationSchema, { FormObject } from "@/utils/validation-schema";
import { useRouter } from "next/navigation";
import { editCoinTransfer } from "@/app/lib/coin-request/action";
import { CoinTransferInterface } from "../type/coin-transfer";
import { useGeneral } from "@/contexts/GeneralContext";
import { RiCoinFill } from "react-icons/ri";

interface CoinTransferTableProps {
  data: CoinTransferInterface[]; // This is required
}

const formFields: FormObject[] = [
  {
    name: "approved_coins",
    label: "No of coins",
    type: "number",
    defaultValue: 0,
    min: 1,
    max: 10,
    required: true,
  },
  {
    name: "status",
    label: "Status",
    type: "radioGroup",
    defaultValue: "approved",
    values: [
      { value: "approved", label: "Approve" },
      { value: "rejected", label: "Reject" },
    ],
    max: 10,
    required: true,
  },
];

interface formValues {
  approved_coins: string;
  status: string;
}

const CoinTransferTable: React.FC<CoinTransferTableProps> = ({ data }: { data: CoinTransferInterface[] }) => {
  // Chakra imports
  const { isOpen, onOpen, onClose } = useDisclosure();
  const { currentUser } = useGeneral();
  const toast = useToast();
  const router = useRouter();

  // Style const
  const brandColor = useColorModeValue("brand.500", "white");
  const modalBg = useColorModeValue("secondaryGray.300", "navy.900");
  const brandStars = useColorModeValue("brand.500", "brand.400");
  const textColor = useColorModeValue("navy.700.700", "white");
  const coinColor = useColorModeValue("yellow.500", "yellow.400");

  // logic values
  const result = data?.length > 0 ? data : [];
  const formRef = useRef(null);
  const [dynamicFormField, setDynamicFormField] = useState(formFields);
  const formDetails = useMemo(() => prepareValidationSchema(dynamicFormField), [dynamicFormField]);
  const [transferForm, setTransferForm] = useState({});
  const [currentEditUser, setCurrentEditUser] = useState<CoinTransferInterface>();
  const { initialFormData, validationSchema } = formDetails;
  const [formValidationSchema, setFormValidationSchema] = useState({});
  const [selectedStatus, setSelectedStatus] = useState("approved");

  useEffect(() => {
    setTransferForm(initialFormData);
    setFormValidationSchema(validationSchema);
  }, [initialFormData, validationSchema]);

  const handleStatusChange = (newValue: string) => {
    setSelectedStatus(newValue);
  };

  // function constant here
  const handleEditClick = async (id: number) => {
    setDynamicFormField((prev) => {
      prev[0] = {
        name: "approved_coins",
        label: "No of coins",
        type: "number",
        defaultValue: 0,
        min: 1,
        max: result[id].requested_coins,
        required: true,
      };
      return [...prev];
    });
    setCurrentEditUser((prev) => {
      return { ...result[id] };
    });
    onOpen();
  };

  const handleSubmitClick = () => {
    formRef?.current?.externalHandleSubmit();
  };

  const handleSubmit = async (values: formValues, data: object) => {
    const payload = {
      approved_coins: parseInt(values.approved_coins),
      status: values.status,
    };
    const res = await editCoinTransfer(payload, currentEditUser._id);
    toast({
      title: values.status == "approved" && res?.status !== 400 ? "200" : "301",
      description: values.status == "approved" && res?.status !== 400 ? "coins request transfer successfully" : res?.message,
      status: values.status == "approved"  && res?.status !== 400? "success" : "error",
      position: "top-right",
      duration: 3000,
      isClosable: true,
    });
    onClose();
    router.refresh();
  };

  const radioOptions = dynamicFormField[1].values as { value: string; label: string }[];

  const action = (id: number) => (
    <Flex gap={2}>
      <Button
        variant="blank"
        onClick={() => {
          handleEditClick(id);
        }}
        alignItems="center"
        justifyContent="center"
        w="37px"
        h="37px"
        lineHeight="100%"
        borderRadius="10px">
        <Icon as={EditIcon} color={brandColor} w="20px" h="20px" />
      </Button>
    </Flex>
  );

  const onChange = (newValue: string) => {};

  return (
    <>
      <DataTable columnsData={columnsDataBlocks} tableData={result as unknown as TableData[]} isSelectable={false} showPageSize={false} actionSlot={action} />

      {currentEditUser && (
        <Modal onClose={onClose} size="lg" isOpen={isOpen}>
          <ModalOverlay />
          <ModalContent bg={modalBg}>
            <ModalHeader display="flex" alignItems="center" gap="2">
              Transfer Coins{" "}
              <Text as="span" display="flex" alignItems="center">
                ( <Icon as={RiCoinFill} color={coinColor} w="20px" h="20px" />
                <Text color={textColor} fontSize="xl" fontWeight="700" lineHeight="100%">
                  {currentUser.coins.toFixed(2)}
                </Text>
                )
              </Text>
            </ModalHeader>
            <ModalCloseButton />
            <ModalBody>
              <Flex justifyContent="space-between">
                <Text mb="36px" color={textColor} fontWeight="400" fontSize="md">
                  Requested By:{" "}
                  <Text as="span" color={textColor} fontSize="md" fontWeight="900">
                    {currentEditUser.requestedFrom.name}
                  </Text>
                </Text>
                <Text mb="36px" color={textColor} fontWeight="400" fontSize="md">
                  Requested Coins:{" "}
                  <Text as="span" color={textColor} fontSize="md" fontWeight="900">
                    {currentEditUser.requested_coins}
                  </Text>
                </Text>
              </Flex>
              <FormComponent
                ref={formRef}
                validationSchema={formValidationSchema}
                initialValues={transferForm}
                submitForm={(formValues, formData) => {
                  handleSubmit(formValues, formData);
                }}>
                <FormControl w="full">
                  <FormLabel></FormLabel>
                  <RadioGroupWrapper form={true} updateOnParent={handleStatusChange} id="status" fieldName="status" defaultValue={"approved"} options={radioOptions}></RadioGroupWrapper>
                  {selectedStatus === "approved" && (
                    <>
                      <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                        Approved Coins<Text color={brandStars}>*</Text>
                      </FormLabel>
                      <InputWrapper form={true} fieldName="approved_coins" defaultValue={"" + currentEditUser.approved_coins} id="approved_coins" placeholder="0" variant="auth" w="full" mb="2px" fontSize="sm" ms={{ base: "0px", md: "0px" }} type="number" maxLength={10} fontWeight="500" />
                    </>
                  )}
                </FormControl>
              </FormComponent>
            </ModalBody>
            <ModalFooter>
              <Button variant="brand" onClick={handleSubmitClick} type="submit" mr={3}>
                Save
              </Button>
              <Button onClick={onClose}>Cancel</Button>
            </ModalFooter>
          </ModalContent>
        </Modal>
      )}
    </>
  );
};

export default CoinTransferTable;
